//+------------------------------------------------------------------+
//|                                                      Cnewton.mqh |
//|                                  Copyright 2024, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2024, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class Cnewton
{
private:

public:
                     Cnewton();
                    ~Cnewton();
   void              Set(vector &W, vector &X, vector &Y);
   void              Get(vector &W, double &XX, double &YY);
   double            Let(vector &X, double Unit = 1.0);
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Cnewton::Cnewton()
{
}
//+------------------------------------------------------------------+
//| INPUT PARAMETERS                                                 |
//|     X   -   vector with x values of sampled data                 |
//|     Y   -   vector with y values of sampled data                 |
//| OUTPUT PARAMETERS                                                |
//|     W   -   vector with coefficients.                            |                                          |
//+------------------------------------------------------------------+
void Cnewton::Set(vector &W, vector &X, vector &Y)
{  vector _w[];
   ArrayResize(_w, int(X.Size() - 1));
   int _x_scale = 1;
   int _y_scale = int(X.Size() - 1);
   for(int i = 0; i < int(X.Size() - 1); i++)
   {  _w[i].Init(_y_scale);
      for(int ii = 0; ii < _y_scale; ii++)
      {  if(X[ii + _x_scale] != X[ii])
         {  if(i == 0)
            {  _w[i][ii] = (Y[ii + 1] - Y[ii]) / (X[ii + _x_scale] - X[ii]);
            }
            else if(i > 0)
            {  _w[i][ii] = (_w[i - 1][ii + 1] - _w[i - 1][ii]) / (X[ii + _x_scale] - X[ii]);
            }
         }
         else
         {  //printf(__FUNCSIG__ + " ERR!, identical X value: " + DoubleToString(X[ii + _x_scale]) + ", at: " + IntegerToString(ii + _x_scale) + ", and: " + IntegerToString(ii));
            return;
         }
      }
      _x_scale++;
      _y_scale--;
      W[i + 1] = _w[i][0];
      if(_y_scale <= 0)
      {  break;
      }
   }
}
//+------------------------------------------------------------------+
//| INPUT PARAMETERS                                                 |
//|     W   -   vector with pre-computed coefficients                |
//|     XX  -   query x value with unknown y                         |
//| OUTPUT PARAMETERS                                                |
//|     YY  -   solution for unknown y.                              |
//+------------------------------------------------------------------+
void Cnewton::Get(vector &W, double &XX, double &YY)
{  YY = W[0];
   for(int i = 1; i < int(W.Size()); i++)
   {  YY += (XX * pow(W[i],i));
   }
}
//+------------------------------------------------------------------+
//| Get Solvability of Shift Inputed Vector.                         |
//+------------------------------------------------------------------+
double Cnewton::Let(vector &X, double Unit = 1.0)
{  X /= Unit;
   matrix _m;
   _m.Init(X.Size(), X.Size());
   for(ulong i = 0; i < X.Size(); i++)
   {  for(ulong ii = 0; ii < X.Size(); ii++)
      {  _m[i][ii] = X[(i + ii) % X.Size()];
      }
   }
   return(_m.Det() * pow(Unit, X.Size()));
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Cnewton::~Cnewton()
{
}
//+------------------------------------------------------------------+
